# Autogenerated file for Kivy Cython configuration
DEF PY3 = 1
DEF USE_RPI = 0
DEF USE_MALI = 0
DEF USE_EGL = 0
DEF USE_OPENGL_ES2 = 0
DEF USE_OPENGL_MOCK = 0
DEF USE_SDL2 = 1
DEF USE_IOS = 0
DEF USE_MESAGL = 0
DEF USE_X11 = 0
DEF USE_WAYLAND = 0
DEF USE_GSTREAMER = 0
DEF USE_AVFOUNDATION = 0
DEF USE_OSX_FRAMEWORKS = 0
DEF DEBUG_GL = 0
DEF DEBUG = False
DEF PLATFORM = "win32"
