# Autogenerated file for Kivy configuration
PY3 = 1
CYTHON_MIN = '0.23'
CYTHON_MAX = '0.28.2'
CYTHON_BAD = '0.27, 0.27.2'
USE_RPI = 0
USE_MALI = 0
USE_EGL = 0
USE_OPENGL_ES2 = 0
USE_OPENGL_MOCK = 0
USE_SDL2 = 1
USE_IOS = 0
USE_MESAGL = 0
USE_X11 = 0
USE_WAYLAND = 0
USE_GSTREAMER = 0
USE_AVFOUNDATION = 0
USE_OSX_FRAMEWORKS = 0
DEBUG_GL = 0
DEBUG = False
PLATFORM = "win32"
